import numpy as np
import numpy.typing as npt

class Vector3:
    mag: float
    mag2: float
    costheta: float
    theta: float
    phi: float
    unit: Vector3
    px: float
    py: float
    pz: float
    def __init__(self, px: float, py: float, pz: float): ...
    def __add__(self, other: Vector3 | int) -> Vector3: ...
    def __radd__(self, other: Vector3 | int) -> Vector3: ...
    def dot(self, other: Vector3) -> float: ...
    def cross(self, other: Vector3) -> Vector3: ...
    def to_numpy(self) -> npt.NDArray[np.float64]: ...

class Vector4:
    mag: float
    mag2: float
    vec3: Vector3
    e: float
    px: float
    py: float
    pz: float
    momentum: Vector3
    gamma: float
    beta: Vector3
    m: float
    m2: float
    def __init__(self, e: float, px: float, py: float, pz: float): ...
    def __add__(self, other: Vector4) -> Vector4: ...
    def boost(self, beta: Vector3) -> Vector4: ...
    def boost_along(self, other: Vector4) -> Vector4: ...
    def to_numpy(self) -> npt.NDArray[np.float64]: ...
    @staticmethod
    def from_momentum(momentum: Vector3, mass: float) -> Vector4: ...



from typing import Literal, overload

from laddu.amplitudes import Amplitude
from laddu.utils.variables import Angles, CosTheta, Phi, Polarization

@overload
def Zlm(
    name: str,
    l: Literal[0],  # noqa: E741
    m: Literal[0],
    r: Literal["+", "plus", "pos", "positive", "-", "minus", "neg", "negative"],
    angles: Angles,
    polarization: Polarization,
) -> Amplitude: ...
@overload
def Zlm(
    name: str,
    l: Literal[1],  # noqa: E741
    m: Literal[-1, 0, 1],
    r: Literal["+", "plus", "pos", "positive", "-", "minus", "neg", "negative"],
    angles: Angles,
    polarization: Polarization,
) -> Amplitude: ...
@overload
def Zlm(
    name: str,
    l: Literal[2],  # noqa: E741
    m: Literal[-2, -1, 0, 1, 2],
    r: Literal["+", "plus", "pos", "positive", "-", "minus", "neg", "negative"],
    angles: Angles,
    polarization: Polarization,
) -> Amplitude: ...
@overload
def Zlm(
    name: str,
    l: Literal[3],  # noqa: E741
    m: Literal[-3, -2, -1, 0, 1, 2, 3],
    r: Literal["+", "plus", "pos", "positive", "-", "minus", "neg", "negative"],
    angles: Angles,
    polarization: Polarization,
) -> Amplitude: ...
@overload
def Zlm(
    name: str,
    l: Literal[4],  # noqa: E741
    m: Literal[-4, -3, -2, -1, 0, 1, 2, 3, 4],
    r: Literal["+", "plus", "pos", "positive", "-", "minus", "neg", "negative"],
    angles: Angles,
    polarization: Polarization,
) -> Amplitude: ...



"""
Usage:
    amptools-to-laddu <input_file> <output_file> [--tree <treename>] [--pol-in-beam | --pol-angle <angle> --pol-magnitude <magnitude>] [-n <num-entries>]
Options:
    --tree <treename>            The tree name in the ROOT file [default: kin].
    --pol-in-beam                Use the beam's momentum for polarization (eps).
    --pol-angle <angle>          The polarization angle in degrees (only used if --pol-in-beam is not used)
    --pol-magnitude <magnitude>  The polarization magnitude (only used if --pol-in-beam is not used)
    -n <num-entries>             Truncate the file to the first n entries for testing.
"""

from __future__ import annotations

from pathlib import Path

import numpy as np
import pandas as pd
import uproot
from docopt import docopt
from loguru import logger


def read_root_file(input_file, tree_name, pol_in_beam, pol_angle, pol_magnitude, num_entries=None):
    """Read ROOT file and extract data with optional polarization from the beam."""
    logger.info(f"Reading ROOT file: {input_file}")
    tfile = uproot.open(input_file)
    tree = tfile[tree_name]
    logger.info(f"Using tree: {tree_name}")
    # Read necessary branches
    E_beam = tree["E_Beam"].array(library="np", entry_stop=num_entries)  # pyright: ignore
    Px_beam = tree["Px_Beam"].array(library="np", entry_stop=num_entries)  # pyright: ignore
    Py_beam = tree["Py_Beam"].array(library="np", entry_stop=num_entries)  # pyright: ignore
    Pz_beam = tree["Pz_Beam"].array(library="np", entry_stop=num_entries)  # pyright: ignore
    weight = tree["Weight"].array(library="np", entry_stop=num_entries) if "Weight" in tree else np.ones_like(E_beam)  # pyright: ignore

    # Final state particles
    E_final = np.array(list(tree["E_FinalState"].array(library="np", entry_stop=num_entries)))  # pyright: ignore
    Px_final = np.array(list(tree["Px_FinalState"].array(library="np", entry_stop=num_entries)))  # pyright: ignore
    Py_final = np.array(list(tree["Py_FinalState"].array(library="np", entry_stop=num_entries)))  # pyright: ignore
    Pz_final = np.array(list(tree["Pz_FinalState"].array(library="np", entry_stop=num_entries)))  # pyright: ignore

    # Handle beam four-vector: (nevents, 4)
    p4_beam = np.stack([E_beam, Px_beam, Py_beam, Pz_beam], axis=-1)

    # Handle final state four-vectors: (nevents, nparticles, 4)
    p4_final = np.stack([E_final, Px_final, Py_final, Pz_final], axis=-1)

    # Check if EPS branch exists and update eps if needed
    if "EPS" in tree:
        logger.info("EPS branch found. Using it for eps values.")
        eps = tree["EPS"].array(library="np", entry_stop=num_entries)  # pyright: ignore
        eps = eps[:, np.newaxis, :]
    if "eps" in tree:
        logger.info("eps branch found. Using it for eps values.")
        eps = tree["eps"].array(library="np", entry_stop=num_entries)  # pyright: ignore
        eps = eps[:, np.newaxis, :]
    elif pol_in_beam:
        logger.info("Using beam's momentum for polarization (eps).")
        eps = np.stack([Px_beam, Py_beam, Pz_beam], axis=-1)[:, np.newaxis]
        # Reset beam momentum
        p4_beam[:, 1:] = 0  # Set Px, Py to 0
        p4_beam[:, 3] = E_beam  # Set Pz = E for beam
    elif pol_angle is not None and pol_magnitude is not None:
        logger.info(f"Using input polarization angle ({pol_angle}) and magnitude ({pol_magnitude}).")
        eps_x = pol_magnitude * np.cos(pol_angle) * np.ones_like(E_beam)
        eps_y = pol_magnitude * np.sin(pol_angle) * np.ones_like(E_beam)
        eps_z = np.zeros_like(E_beam)
        eps = np.stack([eps_x, eps_y, eps_z], axis=-1)[:, np.newaxis]
    else:
        logger.info("Using default or provided eps values.")
        eps = np.zeros((len(E_beam), 1, 3), dtype=np.float32)  # Default to 0

    # Concatenate the beam and final state particles: (nevents, nparticles+1, 4)
    logger.info("Concatenating beam and final state particles.")
    p4s = np.concatenate([p4_beam[:, np.newaxis, :], p4_final], axis=1)

    return p4s.astype(np.float32), weight, eps.astype(np.float32)


def save_as_parquet(p4s, weight, eps, output_file):
    """Save the processed data into Parquet format."""
    logger.info("Saving data to Parquet format.")

    # Flatten the p4s and eps into individual columns
    columns = {}
    n_particles = p4s.shape[1]
    for i in range(n_particles):
        columns[f"p4_{i}_E"] = p4s[:, i, 0]
        columns[f"p4_{i}_Px"] = p4s[:, i, 1]
        columns[f"p4_{i}_Py"] = p4s[:, i, 2]
        columns[f"p4_{i}_Pz"] = p4s[:, i, 3]

    n_eps = eps.shape[1]
    for i in range(n_eps):
        columns[f"eps_{i}_x"] = eps[:, i, 0]
        columns[f"eps_{i}_y"] = eps[:, i, 1]
        columns[f"eps_{i}_z"] = eps[:, i, 2]

    # Add weights
    columns["weight"] = weight

    # Create a DataFrame and save as Parquet
    data = pd.DataFrame(columns)
    data.to_parquet(output_file, index=False)
    logger.info(f"File saved: {output_file}")


def convert_from_amptools(
    input_path: Path,
    output_path: Path,
    tree_name: str = "kin",
    pol_in_beam: bool = False,  # noqa: FBT001, FBT002
    pol_angle_deg: float | None = None,
    pol_magnitude: float | None = None,
    num_entries: int | None = None,
):
    p4s, weight, eps = read_root_file(input_path, tree_name, pol_in_beam, pol_angle_deg, pol_magnitude, num_entries)
    save_as_parquet(p4s, weight, eps, output_path)


def run():
    """Main entry point for the script."""
    args = docopt(__doc__ if __doc__ else "")
    input_file = args["<input_file>"]
    output_file = args["<output_file>"]
    tree_name = args["--tree"]
    pol_in_beam = args["--pol-in-beam"]
    pol_angle = float(args["--pol-angle"]) * np.pi / 180 if args["--pol-angle"] else None
    pol_magnitude = float(args["--pol-magnitude"]) if args["--pol-magnitude"] else None
    num_entries = int(args["-n"]) if args["-n"] else None

    convert_from_amptools(
        Path(input_file), Path(output_file), tree_name, pol_in_beam, pol_angle, pol_magnitude, num_entries
    )

    df_read = pd.read_parquet(output_file)
    print("Output Parquet File (head):")  # noqa: T201
    print(df_read.head())  # noqa: T201
    print("Output Columns:")  # noqa: T201
    for column in df_read.columns:
        print(column)  # noqa: T201



from laddu.laddu import KopfKMatrixA0, KopfKMatrixA2, KopfKMatrixF0, KopfKMatrixF2, KopfKMatrixPi1, KopfKMatrixRho

__all__ = ["KopfKMatrixF0", "KopfKMatrixF2", "KopfKMatrixA0", "KopfKMatrixA2", "KopfKMatrixRho", "KopfKMatrixPi1"]



from laddu.amplitudes import breit_wigner, common, kmatrix, ylm, zlm
from laddu.laddu import (
    Amplitude,
    AmplitudeID,
    Evaluator,
    Expression,
    Manager,
    ParameterLike,
    constant,
    parameter,
)

__all__ = [
    "AmplitudeID",
    "Expression",
    "Amplitude",
    "Manager",
    "Evaluator",
    "ParameterLike",
    "parameter",
    "constant",
    "common",
    "ylm",
    "zlm",
    "breit_wigner",
    "kmatrix",
]



from laddu.laddu import (
    NLL,
    Bound,
    LikelihoodEvaluator,
    LikelihoodExpression,
    LikelihoodID,
    LikelihoodManager,
    LikelihoodScalar,
    LikelihoodTerm,
    Status,
)

__all__ = [
    "NLL",
    "Status",
    "Bound",
    "LikelihoodID",
    "LikelihoodExpression",
    "LikelihoodTerm",
    "LikelihoodManager",
    "LikelihoodEvaluator",
    "LikelihoodScalar",
]



from collections.abc import Sequence
from typing import Any, Literal

import numpy as np
import numpy.typing as npt

from laddu.amplitudes import Evaluator, Expression, Manager
from laddu.data import Dataset

class LikelihoodID:
    def __add__(self, other: LikelihoodID | LikelihoodExpression | int) -> LikelihoodExpression: ...
    def __radd__(self, other: LikelihoodID | LikelihoodExpression | int) -> LikelihoodExpression: ...
    def __mul__(self, other: LikelihoodID | LikelihoodExpression) -> LikelihoodExpression: ...
    def __rmul__(self, other: LikelihoodID | LikelihoodExpression) -> LikelihoodExpression: ...

class LikelihoodExpression:
    def __add__(self, other: LikelihoodID | LikelihoodExpression | int) -> LikelihoodExpression: ...
    def __radd__(self, other: LikelihoodID | LikelihoodExpression | int) -> LikelihoodExpression: ...
    def __mul__(self, other: LikelihoodID | LikelihoodExpression) -> LikelihoodExpression: ...
    def __rmul__(self, other: LikelihoodID | LikelihoodExpression) -> LikelihoodExpression: ...

class LikelihoodTerm: ...

class LikelihoodManager:
    def __init__(self) -> None: ...
    def register(self, likelihood_term: LikelihoodTerm) -> LikelihoodID: ...
    def load(self, likelihood_expression: LikelihoodExpression) -> LikelihoodEvaluator: ...

class LikelihoodEvaluator:
    parameters: list[str]
    def evaluate(self, parameters: list[float] | npt.NDArray[np.float64]) -> float: ...
    def minimize(
        self,
        p0: list[float] | npt.NDArray[np.float64],
        bounds: Sequence[tuple[float | None, float | None]] | None = None,
        method: Literal["lbfgsb", "nelder_mead"] = "lbfgsb",
        max_steps: int = 4000,
        debug: bool = False,  # noqa: FBT001, FBT002
        verbose: bool = False,  # noqa: FBT001, FBT002
        **kwargs,
    ) -> Status: ...

class NLL:
    parameters: list[str]
    data: Dataset
    accmc: Dataset
    def __init__(
        self,
        manager: Manager,
        expression: Expression,
        ds_data: Dataset,
        ds_accmc: Dataset,
    ) -> None: ...
    def as_term(self) -> LikelihoodTerm: ...
    def activate(self, name: str | list[str]) -> None: ...
    def activate_all(self) -> None: ...
    def deactivate(self, name: str | list[str]) -> None: ...
    def deactivate_all(self) -> None: ...
    def isolate(self, name: str | list[str]) -> None: ...
    def evaluate(self, parameters: list[float] | npt.NDArray[np.float64]) -> float: ...
    def project(
        self, parameters: list[float] | npt.NDArray[np.float64], *, mc_evaluator: Evaluator | None = None
    ) -> npt.NDArray[np.float64]: ...
    def project_with(
        self,
        parameters: list[float] | npt.NDArray[np.float64],
        name: str | list[str],
        *,
        mc_evaluator: Evaluator | None = None,
    ) -> npt.NDArray[np.float64]: ...
    def minimize(
        self,
        p0: list[float] | npt.NDArray[np.float64],
        bounds: Sequence[tuple[float | None, float | None]] | None = None,
        method: Literal["lbfgsb", "nelder_mead"] = "lbfgsb",
        max_steps: int = 4000,
        debug: bool = False,  # noqa: FBT001, FBT002
        verbose: bool = False,  # noqa: FBT001, FBT002
        **kwargs,
    ) -> Status: ...

def LikelihoodScalar(name: str) -> LikelihoodTerm: ...  # noqa: N802

class Status:
    x: npt.NDArray[np.float64]
    err: npt.NDArray[np.float64] | None
    x0: npt.NDArray[np.float64]
    fx: float
    cov: npt.NDArray[np.float64] | None
    hess: npt.NDArray[np.float64] | None
    message: str
    converged: bool
    bounds: list[Bound] | None
    n_f_evals: int
    n_g_evals: int

    def __init__(self) -> None: ...
    def save_as(self, path: str) -> None: ...
    @staticmethod
    def load(path: str) -> Status: ...
    def as_dict(self) -> dict[str, Any]: ...
    def __getstate__(self) -> object: ...
    def __setstate__(self, state: object) -> None: ...

class Bound:
    lower: float
    upper: float

__all__ = [
    "NLL",
    "Status",
    "Bound",
    "LikelihoodID",
    "LikelihoodExpression",
    "LikelihoodTerm",
    "LikelihoodManager",
    "LikelihoodEvaluator",
    "LikelihoodScalar",
]



from typing import Literal

import numpy as np
import numpy.typing as npt

from laddu.data import Dataset, Event

class Mass:
    def __init__(self, constituents: list[int]): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...

class CosTheta:
    def __init__(
        self,
        beam: int,
        recoil: list[int],
        daughter: list[int],
        resonance: list[int],
        frame: Literal[
            "Helicity", "HX", "HEL", "GottfriedJackson", "Gottfried Jackson", "GJ", "Gottfried-Jackson"
        ] = "Helicity",
    ): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...

class Phi:
    def __init__(
        self,
        beam: int,
        recoil: list[int],
        daughter: list[int],
        resonance: list[int],
        frame: Literal[
            "Helicity", "HX", "HEL", "GottfriedJackson", "Gottfried Jackson", "GJ", "Gottfried-Jackson"
        ] = "Helicity",
    ): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...

class Angles:
    costheta: CosTheta
    phi: Phi
    def __init__(
        self,
        beam: int,
        recoil: list[int],
        daughter: list[int],
        resonance: list[int],
        frame: Literal[
            "Helicity", "HX", "HEL", "GottfriedJackson", "Gottfried Jackson", "GJ", "Gottfried-Jackson"
        ] = "Helicity",
    ) -> None: ...

class PolAngle:
    def __init__(
        self,
        beam: int,
        recoil: list[int],
    ): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...

class PolMagnitude:
    def __init__(
        self,
        beam: int,
    ): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...

class Polarization:
    pol_magnitude: PolMagnitude
    pol_angle: PolAngle
    def __init__(self, beam: int, recoil: list[int]) -> None: ...

class Mandelstam:
    def __init__(
        self,
        p1: list[int],
        p2: list[int],
        p3: list[int],
        p4: list[int],
        channel: Literal["s", "t", "u"],
    ): ...
    def value(self, event: Event) -> float: ...
    def value_on(self, dataset: Dataset) -> npt.NDArray[np.float64]: ...



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

import laddu

project = "laddu"
copyright = "2024, Nathaniel Dene Hoffman"
author = "Nathaniel Dene Hoffman"
release = laddu.__version__
version = release

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ["sphinx.ext.autodoc", "sphinx.ext.autosummary", "sphinx.ext.napoleon", "sphinx_copybutton"]

autodoc_default_options = {
    "members": True,
    "undoc-members": True,
}

autodoc_member_order = "groupwise"

templates_path = ["_templates"]
exclude_patterns = []


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_static_path = ["_static"]



from laddu.amplitudes import Amplitude, ParameterLike

def Scalar(name: str, value: ParameterLike) -> Amplitude: ...  # noqa: N802
def ComplexScalar(name: str, re: ParameterLike, im: ParameterLike) -> Amplitude: ...  # noqa: N802
def PolarComplexScalar(name: str, r: ParameterLike, theta: ParameterLike) -> Amplitude: ...  # noqa: N802



from typing import Literal

from laddu.amplitudes import Amplitude, ParameterLike
from laddu.utils.variables import Mass

def KopfKMatrixF0(  # noqa: N802
    name: str,
    couplings: tuple[
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
    ],
    channel: Literal[0, 1, 2, 3, 4],
    mass: Mass,
) -> Amplitude: ...
def KopfKMatrixF2(  # noqa: N802
    name: str,
    couplings: tuple[
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
    ],
    channel: Literal[0, 1, 2, 3],
    mass: Mass,
) -> Amplitude: ...
def KopfKMatrixA0(  # noqa: N802
    name: str,
    couplings: tuple[
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
    ],
    channel: Literal[0, 1],
    mass: Mass,
) -> Amplitude: ...
def KopfKMatrixA2(  # noqa: N802
    name: str,
    couplings: tuple[
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
    ],
    channel: Literal[0, 1, 2],
    mass: Mass,
) -> Amplitude: ...
def KopfKMatrixRho(  # noqa: N802
    name: str,
    couplings: tuple[
        tuple[ParameterLike, ParameterLike],
        tuple[ParameterLike, ParameterLike],
    ],
    channel: Literal[0, 1, 2],
    mass: Mass,
) -> Amplitude: ...
def KopfKMatrixPi1(  # noqa: N802
    name: str,
    couplings: tuple[tuple[ParameterLike, ParameterLike],],
    channel: Literal[0, 1],
    mass: Mass,
) -> Amplitude: ...



from abc import ABCMeta, abstractmethod

from laddu.amplitudes import Manager, constant, parameter
from laddu.amplitudes.breit_wigner import BreitWigner
from laddu.amplitudes.common import ComplexScalar, PolarComplexScalar, Scalar
from laddu.amplitudes.ylm import Ylm
from laddu.amplitudes.zlm import Zlm
from laddu.convert import convert_from_amptools
from laddu.data import BinnedDataset, Dataset, open
from laddu.likelihoods import NLL, LikelihoodManager, Status
from laddu.utils.variables import Angles, CosTheta, Mandelstam, Mass, Phi, PolAngle, Polarization, PolMagnitude
from laddu.utils.vectors import Vector3, Vector4

from . import amplitudes, convert, data, likelihoods, utils
from .laddu import version

__version__ = version()


class Observer(metaclass=ABCMeta):
    @abstractmethod
    def callback(self, step: int, status: Status) -> tuple[Status, bool]:
        pass


__all__ = [
    "__version__",
    "convert",
    "convert_from_amptools",
    "Dataset",
    "open",
    "BinnedDataset",
    "utils",
    "data",
    "amplitudes",
    "likelihoods",
    "Vector3",
    "Vector4",
    "CosTheta",
    "Phi",
    "Angles",
    "PolMagnitude",
    "PolAngle",
    "Polarization",
    "Mandelstam",
    "Mass",
    "Manager",
    "LikelihoodManager",
    "NLL",
    "Status",
    "Observer",
    "parameter",
    "constant",
    "Scalar",
    "ComplexScalar",
    "PolarComplexScalar",
    "Ylm",
    "Zlm",
    "BreitWigner",
]



from laddu.laddu import Angles, CosTheta, Mandelstam, Mass, Phi, PolAngle, Polarization, PolMagnitude

__all__ = ["Angles", "CosTheta", "Mass", "Phi", "PolAngle", "Polarization", "PolMagnitude", "Mandelstam"]



import numpy as np
import numpy.typing as npt

from laddu.amplitudes import breit_wigner, common, kmatrix, ylm, zlm
from laddu.data import Dataset

class ParameterLike: ...

def parameter(name: str) -> ParameterLike: ...
def constant(value: float) -> ParameterLike: ...

class AmplitudeID:
    def real(self) -> Expression: ...
    def imag(self) -> Expression: ...
    def norm_sqr(self) -> Expression: ...
    def __add__(self, other: AmplitudeID | Expression | int) -> Expression: ...
    def __radd__(self, other: AmplitudeID | Expression | int) -> Expression: ...
    def __mul__(self, other: AmplitudeID | Expression) -> Expression: ...
    def __rmul__(self, other: AmplitudeID | Expression) -> Expression: ...

class Expression:
    def real(self) -> Expression: ...
    def imag(self) -> Expression: ...
    def norm_sqr(self) -> Expression: ...
    def __add__(self, other: AmplitudeID | Expression | int) -> Expression: ...
    def __radd__(self, other: AmplitudeID | Expression | int) -> Expression: ...
    def __mul__(self, other: AmplitudeID | Expression) -> Expression: ...
    def __rmul__(self, other: AmplitudeID | Expression) -> Expression: ...

class Amplitude: ...

class Manager:
    def __init__(self) -> None: ...
    def register(self, amplitude: Amplitude) -> AmplitudeID: ...
    def load(self, expression: Expression, dataset: Dataset) -> Evaluator: ...

class Evaluator:
    parameters: list[str]
    def activate(self, name: str | list[str]) -> None: ...
    def activate_all(self) -> None: ...
    def deactivate(self, name: str | list[str]) -> None: ...
    def deactivate_all(self) -> None: ...
    def isolate(self, name: str | list[str]) -> None: ...
    def evaluate(self, parameters: list[float] | npt.NDArray[np.float64]) -> npt.NDArray[np.complex128]: ...

__all__ = [
    "AmplitudeID",
    "Expression",
    "Amplitude",
    "Manager",
    "Evaluator",
    "ParameterLike",
    "parameter",
    "constant",
    "common",
    "ylm",
    "zlm",
    "breit_wigner",
    "kmatrix",
]
